unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls;

type
  TMainForm = class(TForm)
    CountdownTimer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClick(Sender: TObject);
    procedure CountdownTimerTimer(Sender: TObject);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormPaint(Sender: TObject);
  private
    { Private declarations }
    PaintData:array of record
                         BaseY,SizeY:integer;
                         Style:TFontStyles;
                         DrawBar:boolean;
                         Text:string;
                       end;
  public
    { Public declarations }
    CloseCountdown:integer;
    procedure UpdateCaption;
  end;

var
  MainForm: TMainForm;

implementation

uses birthd_core;

{$R *.dfm}

var LC_WINDOW_TITLE:string='Happy Birthday! [$info]';
    LC_SECONDS:string='$secs seconds to go';
    LC_ONE_SECOND:string='1 second to go';
    LC_CLOSE_MANUALLY:string='click to close';
    SecondsPerBirthday:integer=5;
    BarColorR:integer=170;
    BarColorG:integer=204;
    BarColorB:integer=240;
    MarginX:integer=2;
    MarginY:integer=2;
    LineSpacing:integer=4;
    BarPadding:integer=2;
    //
    EndX:integer;

function AdditionalConfig(const Key,ValueS:string;ValueI:integer;IsInt:boolean):boolean;
begin
  AdditionalConfig:=false;
       if Key='LC_WINDOW_TITLE' then LC_WINDOW_TITLE:=ValueS
  else if Key='LC_SECONDS' then LC_SECONDS:=ValueS
  else if Key='LC_ONE_SECOND' then LC_ONE_SECOND:=ValueS
  else if Key='LC_CLOSE_MANUALLY' then LC_CLOSE_MANUALLY:=ValueS
  else if Key='Win32.FontName' then MainForm.Font.Name:=ValueS
  else if Key='Win32.FontSize' then
          begin if IsInt then MainForm.Font.Size:=ValueI else exit; end
  else if Key='SecondsPerBirthday' then
          begin if IsInt then SecondsPerBirthday:=ValueI else exit; end
  else if Key='BarColorR' then
          begin if IsInt then BarColorR:=ValueI else exit; end
  else if Key='BarColorG' then
          begin if IsInt then BarColorG:=ValueI else exit; end
  else if Key='BarColorB' then
          begin if IsInt then BarColorB:=ValueI else exit; end
  else if Key='MarginX' then
          begin if IsInt then MarginX:=ValueI else exit; end
  else if Key='MarginY' then
          begin if IsInt then MarginY:=ValueI else exit; end
  else if Key='LineSpacing' then
          begin if IsInt then LineSpacing:=ValueI else exit; end
  else if Key='BarPadding' then
          begin if IsInt then BarPadding:=ValueI else exit; end;
  AdditionalConfig:=true;
end;

var ParseErrors:boolean=false;
procedure MyParserMessageProc(const msg:string);
begin
  if not ParseErrors then AllocConsole;
  ParseErrors:=true;
  writeln(msg);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  ParserMessageProc:=MyParserMessageProc;
  ParserNotifyConfigProc:=AdditionalConfig;
  Parse('.birthd');
  SetLength(PaintData,0);
  Top:=0; Left:=0;
  ClientWidth:=200; ClientHeight:=0;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  Finalize(PaintData);
  Finalize(Birthdays);
end;

procedure TMainForm.FormShow(Sender: TObject);
var i:integer;
    Extent:TSize;
    CurrY:integer;
begin
  if not(ParseErrors) AND (length(Birthdays)<1) then begin Close; exit; end;
  SetLength(PaintData,length(Birthdays));

  CurrY:=MarginY; EndX:=0;
  for i:=0 to High(Birthdays) do with Canvas,PaintData[i],Birthdays[i] do begin
    BaseY:=CurrY; Text:=Line;
    DrawBar:=(Diff=0);
    if DrawBar then Style:=[fsBold] else Style:=[];
    Font.Style:=Style;
    Extent:=TextExtent(Text);
    SizeY:=Extent.cy;
    if Extent.cx>EndX then EndX:=Extent.cx;
    inc(CurrY,SizeY+LineSpacing);
  end;
  inc(EndX,MarginX);
  ClientWidth:=EndX+MarginX;
  ClientHeight:=CurrY-LineSpacing+MarginY;

  CloseCountdown:=length(Birthdays)*SecondsPerBirthday;
  CountdownTimer.Enabled:=true;
  UpdateCaption;
end;

procedure TMainForm.FormClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.UpdateCaption;
var Info:string;
begin
  case CloseCountdown of
    -1:Info:=LC_CLOSE_MANUALLY;
    0:begin Close; exit; end;
    1:Info:=LC_ONE_SECOND;
  else
    Info:=Trim(StringReplace(LC_SECONDS,'$secs',IntToStr(CloseCountdown),[]));
  end;
  Caption:=Trim(StringReplace(LC_WINDOW_TITLE,'$info',Info,[]));
end;

procedure TMainForm.CountdownTimerTimer(Sender: TObject);
begin
  dec(CloseCountdown);
  UpdateCaption;
end;

procedure TMainForm.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if CloseCountdown<0 then exit;
  CountdownTimer.Enabled:=false;
  CloseCountdown:=-1;
  UpdateCaption;
end;

procedure TMainForm.FormPaint(Sender: TObject);
var i:integer;
begin
  for i:=0 to High(PaintData) do with Canvas,PaintData[i] do begin
    if DrawBar then begin
      Brush.Color:=RGB(BarColorR,BarColorG,BarColorB);
      FillRect(Rect(MarginX-BarPadding,BaseY-BarPadding,EndX+BarPadding,BaseY+SizeY+BarPadding));
    end else
      Brush.Color:=Color;
    Font.Style:=Style;
    TextOut(MarginX,BaseY,Text);
  end;
end;

end.
